/*****************************************************************************************
   
   Copyright (c) Microsoft Corporation. All rights reserved.

   Use of this code sample is subject to the terms of the Microsoft
   Public License, a copy of which should always be distributed with
   this file.  You can also access a copy of this license agreement at:
   http://www.microsoft.com/opensource/licenses.mspx#Ms-PL

 ****************************************************************************************/
using System;

namespace Microsoft.CSA.FeedSync.Utilities
{
    class FeedSyncDelete
    {
        static string s_SourcePath;
        static string s_ID;
        static string s_By = "FeedSyncDelete.exe";

        static void Main(string[] i_Arguments)
        {
            try
            {
                //  Validate arguments
                if (i_Arguments.Length < 2)
                {
                    Microsoft.CSA.FeedSync.Utilities.FeedSyncDelete.DisplayUsage(null);
                    return;
                }

                //  Get required parameters
                s_SourcePath = i_Arguments[0];
                s_ID = i_Arguments[1];

                //  Get optional parameters
                if (i_Arguments.Length > 2)
                    s_By = i_Arguments[2];

                //  Create new instance of xml document
                System.Xml.XmlDocument SourceXmlDocument = new System.Xml.XmlDocument();

                try
                {
                    //  Load the feed contents into xml document
                    SourceXmlDocument.Load(s_SourcePath);
                }
                catch (System.Exception Exception)
                {
                    System.Console.WriteLine("Exception while loading '" + s_SourcePath + "': " + Exception.Message);
                    return;
                }

                //  Create an instance of Feed
                Microsoft.CSA.FeedSync.FeedSyncLibrary.RSSFeed RSSFeed = new Microsoft.CSA.FeedSync.FeedSyncLibrary.RSSFeed(SourceXmlDocument);

                //  Get existing node
                Microsoft.CSA.FeedSync.FeedSyncLibrary.RSSFeedSyncItem RSSFeedSyncItem;
                if (!RSSFeed.FindItem(s_ID, out RSSFeedSyncItem))
                {
                    System.Console.WriteLine("Specified item does not exist!");
                    return;
                }

                //  Delete existing node
                RSSFeedSyncItem.Delete
                    (
                    System.DateTime.Now,
                    s_By
                    );

                //  Output updated feed
                System.Console.WriteLine(RSSFeed.XmlDocument.OuterXml);
            }
            catch (System.Exception Exception)
            {
                System.Console.WriteLine("Exception encountered: " + Exception.Message);
            }
        }

        static void DisplayUsage(string i_Text)
        {
            string Text = "Usage:\r\nFeedSyncDelete [SourcePath] [ID] [By]\r\n\r\nParameters:\r\n  SourcePath=fully qualified filename of original FeedSync document\r\n  GUID=unique value for new item\r\n  By=unique entity making the change (optional - default is \"\")\r\n";

            //  If text was provided, prepend it to default usage text
            if (!System.String.IsNullOrEmpty(i_Text))
                Text = i_Text + "\r\n\r\n" + Text;

            System.Console.WriteLine(Text);
        }
    }
}
