﻿using System;
using System.Linq;

namespace Microsoft.CSA.FeedSync.FeedSyncLibrarySL
{
    public class Sync
    {
        private Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem m_IFeedSyncItem;

        private System.Xml.Linq.XElement m_XElement;

        private string m_ID;

        private uint m_Updates;

        private bool m_NoConflicts = false;
        private bool m_Deleted = false;

        private System.Collections.Generic.Dictionary<string, Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem> m_ConflictIFeedSyncItemDictionary = new System.Collections.Generic.Dictionary<string, Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem>();

        private System.Collections.Generic.List<Microsoft.CSA.FeedSync.FeedSyncLibrarySL.History> m_HistoryList = new System.Collections.Generic.List<Microsoft.CSA.FeedSync.FeedSyncLibrarySL.History>();

        private System.Xml.Linq.XElement m_ConflictsNodeXElement = null;

        public enum CompareResult
        {
            Newer,
            Older,
            EqualWithSameConflictData,
            EqualWithDifferentConflictData
        }

        static public System.Collections.Generic.IList<T> Merge<T>(System.Collections.Generic.IList<T> i_LocalItemList, System.Collections.Generic.IList<T> i_IncomingItemList) where T : Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem, new()
        {
            System.Collections.Generic.List<T> MergedItemList = new System.Collections.Generic.List<T>();

            System.Collections.Generic.Dictionary<string, T> LocalItemDictionary = new System.Collections.Generic.Dictionary<string, T>();
            foreach (T LocalItem in i_LocalItemList)
                LocalItemDictionary[LocalItem.Sync.ID] = LocalItem;

            foreach (T IncomingItem in i_IncomingItemList)
            {
                T MergedItem = IncomingItem;

                if (LocalItemDictionary.ContainsKey(IncomingItem.Sync.ID))
                {
                    T LocalItem = LocalItemDictionary[IncomingItem.Sync.ID];
                    Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.CompareResult CompareResult = LocalItem.Sync.CompareTo(IncomingItem.Sync);

                    bool NeedToMerge =
                        (CompareResult == Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.CompareResult.EqualWithDifferentConflictData) ||
                        (CompareResult == Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.CompareResult.Older) ||
                        ((CompareResult == Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.CompareResult.Newer) &&
                          (!IncomingItem.Sync.IsSubsumedBy(LocalItem.Sync)));

                    if (!NeedToMerge)
                        continue;

                    MergedItem = Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.Merge<T>(LocalItem, IncomingItem);
                }

                MergedItemList.Add(MergedItem);
            }

            return MergedItemList;
        }

        static public T Merge<T>(T i_LocalItem, T i_IncomingItem) where T : Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem, new()
        {
            if (i_LocalItem == null)
                throw new System.ArgumentException("i_LocalItem must be non-null!");

            if (i_LocalItem.XElement == null)
                throw new System.ArgumentException("i_LocalItem.XElement must be non-null!");

            if (i_LocalItem.Sync == null)
                throw new System.ArgumentException("i_LocalItem.Sync must be non-null!");

            if (i_IncomingItem == null)
                throw new System.ArgumentException("i_IncomingItem must be non-null!");

            if (i_IncomingItem.XElement == null)
                throw new System.ArgumentException("i_IncomingItem.XElement must be non-null!");

            if (i_IncomingItem.Sync == null)
                throw new System.ArgumentException("i_IncomingItem.Sync must be non-null!");

            if (i_LocalItem.Sync.ID != i_IncomingItem.Sync.ID)
                throw new System.ArgumentException("Sync ID values must match!");

            if (i_LocalItem.Sync.NoConflicts != i_IncomingItem.Sync.NoConflicts)
                throw new System.ArgumentException("Sync NoConflicts values must match!");

            object ClonedItem = i_LocalItem.Clone();
            if (!(ClonedItem is T))
                throw new System.Exception("Cloned LocalItem is not same type as original item!");

            T ClonedLocalItem = (T)ClonedItem;
            if (ClonedLocalItem.Sync == i_LocalItem.Sync)
                throw new System.Exception("Cloned LocalItem can't have same Sync reference!");

            if (ClonedLocalItem.XElement == i_LocalItem.XElement)
                throw new System.Exception("Cloned LocalItem can't have same XElement reference!");

            ClonedItem = i_IncomingItem.Clone();
            if (!(ClonedItem is T))
                throw new System.Exception("Cloned IncomingItem is not same type as original item!");

            T ClonedIncomingItem = (T)ClonedItem;
            if (ClonedIncomingItem.Sync == i_IncomingItem.Sync)
                throw new System.Exception("Cloned IncomingItem can't have same Sync reference!");

            if (ClonedIncomingItem.XElement == i_IncomingItem.XElement)
                throw new System.Exception("Cloned IncomingItem can't have same XElement reference!");

            if (ClonedLocalItem.Sync == ClonedIncomingItem.Sync)
                throw new System.Exception("Cloned LocalItem and cloned IncomingItem can't have same Sync reference!");

            if (ClonedLocalItem.XElement == ClonedIncomingItem.XElement)
                throw new System.Exception("Cloned LocalItem and cloned IncomingItem can't have same XElement reference!");

            System.Collections.Generic.List<T> LocalItemList = new System.Collections.Generic.List<T>();
            LocalItemList.Add(ClonedLocalItem);

            System.Collections.Generic.List<T> IncomingItemList = new System.Collections.Generic.List<T>();
            IncomingItemList.Add(ClonedIncomingItem);

            //  Perform conflict feed item processing (if necessary)
            if (!ClonedIncomingItem.Sync.NoConflicts)
            {
                foreach (Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem LocalConflictIFeedSyncItem in ClonedLocalItem.Sync.Conflicts)
                {
                    System.Xml.Linq.XElement ClonedLocalConflictItemXElement = new System.Xml.Linq.XElement(LocalConflictIFeedSyncItem.XElement);
                    T LocalConflictItem = new T();

                    Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync LocalSync = LocalConflictItem.Initialize(ClonedLocalConflictItemXElement);

                    if (LocalConflictItem.Sync == null)
                        throw new System.Exception("LocalConflictItem.Sync must be non-null!");

                    LocalItemList.Add(LocalConflictItem);
                }

                foreach (Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem IncomingConflictIFeedSyncItem in ClonedIncomingItem.Sync.Conflicts)
                {
                    T IncomingConflictItem = new T();
                    System.Xml.Linq.XElement ClonedIncomingConflictItemXElement = new System.Xml.Linq.XElement(IncomingConflictIFeedSyncItem.XElement);

                    Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync IncomingSync = IncomingConflictItem.Initialize(ClonedIncomingConflictItemXElement);

                    if (IncomingConflictItem.Sync == null)
                        throw new System.Exception("IncomingConflictItem.Sync must be non-null!");

                    IncomingItemList.Add(IncomingConflictItem);
                }
            }

            ClonedLocalItem.Sync.RemoveAllConflictItemNodes();
            ClonedIncomingItem.Sync.RemoveAllConflictItemNodes();

            T WinnerItem = default(T);
            System.Collections.Generic.List<T> MergedItemList = new System.Collections.Generic.List<T>();

            #region Process local list

            //  Iterate local items first, looking for subsumption
            for (int Index = LocalItemList.Count - 1; Index >= 0; --Index)
            {
                T LocalItem = LocalItemList[Index];
                bool Subsumed = false;

                foreach (T IncomingItem in IncomingItemList)
                {
                    if (LocalItem.Sync.IsSubsumedBy(IncomingItem.Sync))
                    {
                        Subsumed = true;
                        break;
                    }
                }

                if (Subsumed)
                {
                    LocalItemList.RemoveAt(Index);
                    continue;
                }

                MergedItemList.Add(LocalItem);

                if (WinnerItem == null)
                    WinnerItem = LocalItem;
                else
                {
                    Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.CompareResult CompareResult = WinnerItem.Sync.CompareTo(LocalItem.Sync);

                    if (Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.CompareResult.Newer != CompareResult)
                        WinnerItem = LocalItem;
                }
            }

            #endregion

            #region Process incoming feeditem node list

            //  Iterate incoming items next, looking for subsumption
            for (int Index = IncomingItemList.Count - 1; Index >= 0; --Index)
            {
                T IncomingItem = IncomingItemList[Index];

                bool Subsumed = false;

                foreach (T LocalItem in LocalItemList)
                {
                    if (IncomingItem.Sync.IsSubsumedBy(LocalItem.Sync))
                    {
                        Subsumed = true;
                        break;
                    }
                }

                if (Subsumed)
                {
                    IncomingItemList.RemoveAt(Index);
                    continue;
                }

                MergedItemList.Add(IncomingItem);

                if (WinnerItem == null)
                    WinnerItem = IncomingItem;
                else
                {
                    Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.CompareResult CompareResult = WinnerItem.Sync.CompareTo(IncomingItem.Sync);

                    if (Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.CompareResult.Newer != CompareResult)
                        WinnerItem = IncomingItem;
                }
            }

            #endregion

            foreach (T ConflictItem in MergedItemList)
            {
                if (!WinnerItem.Equals(ConflictItem))
                    WinnerItem.Sync.AddConflictItem<T>(ConflictItem);
            }

            return WinnerItem;
        }

        public Sync(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem i_IFeedSyncItem)
        {
            if (i_IFeedSyncItem == null)
                throw new System.ArgumentException("i_IFeedSyncItem must be non-null!");

            if (i_IFeedSyncItem.XElement == null)
                throw new System.ArgumentException("i_IFeedSyncItem.XElement must be non-null!");

            System.Xml.Linq.XName SyncXName = Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.FeedSyncXNamespace + Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.SYNC_ELEMENT_NAME;
            System.Xml.Linq.XElement SyncXElement = (System.Xml.Linq.XElement)i_IFeedSyncItem.XElement.Element(SyncXName);
            if (SyncXElement == null)
                throw new System.ArgumentException("Invalid item - does not contain a FeedSync sync element!");

            this.InitializeFromXElement(i_IFeedSyncItem, SyncXElement);
        }

        public Sync(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem i_IFeedSyncItem, System.Xml.Linq.XElement i_SyncXElement)
        {
            if (i_IFeedSyncItem == null)
                throw new System.ArgumentException("i_IFeedSyncItem must be non-null!");

            if (i_IFeedSyncItem.XElement == null)
                throw new System.ArgumentException("i_IFeedSyncItem.XElement must be non-null!");

            if (i_SyncXElement == null)
                throw new System.ArgumentException("i_SyncXElement must be non-null!");

            if (i_SyncXElement.Document != i_IFeedSyncItem.XElement.Document)
                i_SyncXElement = new System.Xml.Linq.XElement(i_SyncXElement);

            this.InitializeFromXElement(i_IFeedSyncItem, i_SyncXElement);
        }

        public Sync(System.Xml.Linq.XDocument i_XDocument, string i_By, string i_ID)
            : this(i_XDocument, i_By, i_ID, 0, false, false)
        {
        }

        public Sync(System.Xml.Linq.XDocument i_XDocument, string i_By, string i_ID, uint i_Updates, bool i_Deleted, bool i_NoConflicts)
        {
            if (i_XDocument == null)
                throw new System.ArgumentException("i_XDocument must be non-null!");

            m_ID = i_ID;
            m_Updates = i_Updates;
            m_Deleted = i_Deleted;
            m_NoConflicts = i_NoConflicts;
            m_HistoryList = new System.Collections.Generic.List<History>();
            m_ConflictIFeedSyncItemDictionary = new System.Collections.Generic.Dictionary<string, IFeedSyncItem>();

            m_XElement = new System.Xml.Linq.XElement(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.FeedSyncXNamespace + Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.SYNC_ELEMENT_NAME);

            m_XElement.SetAttributeValue(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.ID_ATTRIBUTE, m_ID);
            m_XElement.SetAttributeValue(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.UPDATES_ATTRIBUTE, m_Updates.ToString());

            if (m_Deleted)
                m_XElement.SetAttributeValue(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.DELETED_ATTRIBUTE, "true");

            if (m_NoConflicts)
                m_XElement.SetAttributeValue(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.NO_CONFLICTS_ATTRIBUTE, "true");

            this.Update
                (
                System.DateTime.Now,
                i_By
                );

            this.Validate();
        }

        private void InitializeFromXElement(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem i_IFeedSyncItem, System.Xml.Linq.XElement i_SyncXElement)
        {
            bool InvalidElement =
                (i_SyncXElement.Name.LocalName != Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.SYNC_ELEMENT_NAME) ||
                (i_SyncXElement.Name.NamespaceName != Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.FEEDSYNC_XML_NAMESPACE_URI);

            if (InvalidElement)
                throw new System.ArgumentException("Invalid i_SyncXElement!");

            m_IFeedSyncItem = i_IFeedSyncItem;
            m_XElement = i_SyncXElement;

            m_ID = m_XElement.Attribute(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.ID_ATTRIBUTE).Value;
            m_Updates = System.Convert.ToUInt32(m_XElement.Attribute(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.UPDATES_ATTRIBUTE).Value);

            if (m_XElement.Attribute(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.DELETED_ATTRIBUTE) != null)
                m_Deleted = (m_XElement.Attribute(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.DELETED_ATTRIBUTE).Value == "true");

            if (m_XElement.Attribute(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.NO_CONFLICTS_ATTRIBUTE) != null)
                m_NoConflicts = (m_XElement.Attribute(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.NO_CONFLICTS_ATTRIBUTE).Value == "true");

            System.Xml.Linq.XName HistoryXName = Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.FeedSyncXNamespace + Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.HISTORY_ELEMENT_NAME;
            var HistoryXElements = m_XElement.Elements(HistoryXName);

            foreach (System.Xml.Linq.XElement HistoryXElement in HistoryXElements)
            {
                Microsoft.CSA.FeedSync.FeedSyncLibrarySL.History History = new Microsoft.CSA.FeedSync.FeedSyncLibrarySL.History(HistoryXElement);

                m_HistoryList.Add(History);
            }

            if (!m_NoConflicts)
            {
                System.Xml.Linq.XName ConflictsXName = Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.FeedSyncXNamespace + Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.CONFLICTS_ELEMENT_NAME;
                m_ConflictsNodeXElement = i_SyncXElement.Element(ConflictsXName);

                if (m_ConflictsNodeXElement != null)
                {
                    System.Xml.Linq.XName SyncXName = Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.FeedSyncXNamespace + Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.SYNC_ELEMENT_NAME;
                    var ConflictItemXElements =
                        from ConflictItemXElement in m_ConflictsNodeXElement.Elements()
                        where (ConflictItemXElement.Element(SyncXName) != null)
                        select ConflictItemXElement;

                    System.Type ConflictItemType = i_IFeedSyncItem.GetType();

                    foreach (System.Xml.Linq.XElement ConflictItemXElement in ConflictItemXElements)
                    {
                        Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem ConflictFeedSyncItem = (Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem)System.Activator.CreateInstance(ConflictItemType);

                        ConflictFeedSyncItem.Initialize(ConflictItemXElement);

                        string Key = System.String.Format
                            (
                            "{0}{1}{2}",
                            ConflictFeedSyncItem.Sync.Updates,
                            ConflictFeedSyncItem.Sync.TopMostHistory.Sequence,
                            ConflictFeedSyncItem.Sync.TopMostHistory.By
                            );

                        if (ConflictFeedSyncItem.Sync.TopMostHistory.WhenDateTime != null)
                            Key += ((System.DateTime)ConflictFeedSyncItem.Sync.TopMostHistory.WhenDateTime);

                        if (!m_ConflictIFeedSyncItemDictionary.ContainsKey(Key))
                            m_ConflictIFeedSyncItemDictionary.Add(Key, ConflictFeedSyncItem);
                    }
                }
            }

            this.Validate();
        }

        private void Validate()
        {
            if (System.String.IsNullOrEmpty(m_ID))
                throw new System.ArgumentException("Invalid ID!");

            if ((m_Updates < 1) || (m_Updates > (System.Math.Pow(2, 32) - 1)))
                throw new System.ArgumentException("Invalid Updates!");

            if (m_HistoryList == null)
                throw new System.ArgumentException("HistoryList must be non-null!");

            if (m_ConflictIFeedSyncItemDictionary == null)
                throw new System.ArgumentException("ConflictIFeedSyncItemDictionary must be non-null!");
        }

        public Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.CompareResult CompareTo(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync i_IncomingSync)
        {
            if (i_IncomingSync.Updates > m_Updates)
                return Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.CompareResult.Older;

            if (i_IncomingSync.Updates == m_Updates)
            {
                if (!this.TopMostHistory.WhenDateTime.HasValue && i_IncomingSync.TopMostHistory.WhenDateTime.HasValue)
                    return Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.CompareResult.Older;

                int CompareOrdinalResult = 0;

                if (this.TopMostHistory.WhenDateTime.HasValue && i_IncomingSync.TopMostHistory.WhenDateTime.HasValue)
                {
                    //  --------------------------------------------------------------------------------------------------
                    //  BIG HONKING NOTE:  Since dates are normalized to RFC3339 form, just compare string values to avoid
                    //                     .NET bugs with DateTime[Offset] comparisons
                    //  --------------------------------------------------------------------------------------------------
                    string LocalSyncWhen = this.TopMostHistory.WhenDateTime.Value.ToString(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.DATE_STRING_FORMAT);
                    string IncomingSyncWhen = i_IncomingSync.TopMostHistory.WhenDateTime.Value.ToString(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.DATE_STRING_FORMAT);

                    CompareOrdinalResult = System.String.CompareOrdinal(LocalSyncWhen, IncomingSyncWhen);
                    if (CompareOrdinalResult < 0)
                        return Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.CompareResult.Older;

                    if (CompareOrdinalResult > 0)
                        return Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.CompareResult.Newer;
                }

                CompareOrdinalResult = System.String.CompareOrdinal(this.TopMostHistory.By, i_IncomingSync.TopMostHistory.By);
                if (CompareOrdinalResult > 0)
                    return Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.CompareResult.Older;

                if (CompareOrdinalResult < 0)
                    return Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.CompareResult.Newer;

                if (m_ConflictIFeedSyncItemDictionary.Count == i_IncomingSync.Conflicts.Count)
                {
                    if (m_ConflictIFeedSyncItemDictionary.Count > 0)
                    {
                        foreach (Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem ConflictIFeedSyncItem in m_ConflictIFeedSyncItemDictionary.Values)
                        {
                            bool MatchingConflictItem = false;

                            foreach (Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem IncomingConflictIFeedSyncItem in i_IncomingSync.Conflicts)
                            {
                                if (IncomingConflictIFeedSyncItem.Sync.CompareTo(ConflictIFeedSyncItem.Sync) == Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.CompareResult.EqualWithSameConflictData)
                                {
                                    MatchingConflictItem = true;
                                    break;
                                }
                            }

                            if (!MatchingConflictItem)
                                return Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.CompareResult.EqualWithDifferentConflictData;
                        }
                    }

                    return Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.CompareResult.EqualWithSameConflictData;
                }
                else
                {
                    return Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.CompareResult.EqualWithDifferentConflictData;
                }
            }

            return Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync.CompareResult.Newer;
        }

        public bool IsSubsumedBy(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Sync i_IncomingSync)
        {
            bool IsSubsumed = false;

            //  Compare local item topmost history to incoming item history
            foreach (Microsoft.CSA.FeedSync.FeedSyncLibrarySL.History IncomingHistory in i_IncomingSync.Histories)
            {
                if (this.TopMostHistory.IsSubsumedBy(IncomingHistory))
                {
                    IsSubsumed = true;
                    break;
                }
            }

            if (!IsSubsumed)
            {
                //  Compare local item incoming item conflict
                foreach (Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem IncomingConflictIFeedSyncItem in i_IncomingSync.Conflicts)
                {
                    if (this.IsSubsumedBy(IncomingConflictIFeedSyncItem.Sync))
                    {
                        IsSubsumed = true;
                        break;
                    }
                }
            }

            if (!IsSubsumed)
                return false;


            foreach (Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem ConflictIFeedSyncItem in this.Conflicts)
            {
                IsSubsumed = false;

                //  Compare local item conflict to incoming item
                if (ConflictIFeedSyncItem.Sync.IsSubsumedBy(i_IncomingSync))
                {
                    IsSubsumed = true;
                    continue;
                }

                // Compare local item conflict to incoming item conflict
                foreach (Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem IncomingConflictIFeedSyncItem in i_IncomingSync.Conflicts)
                {
                    if (ConflictIFeedSyncItem.Sync.IsSubsumedBy(IncomingConflictIFeedSyncItem.Sync))
                    {
                        IsSubsumed = true;
                        break;
                    }
                }

                if (!IsSubsumed)
                    return false;
            }

            return true;
        }

        public void RemoveAllConflictItemNodes()
        {
            if (m_ConflictIFeedSyncItemDictionary.Count == 0)
                return;

            //  Delete "sx:conflicts" element
            m_ConflictsNodeXElement.Remove();
            m_ConflictsNodeXElement = null;

            //  Empty node list
            m_ConflictIFeedSyncItemDictionary.Clear();
        }

        public void AddConflictItem<T>(T i_ConflictFeedSyncItem) where T : Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem, new()
        {
            if (i_ConflictFeedSyncItem.XElement.Document != m_XElement.Document)
            {
                System.Xml.Linq.XElement ImportedConflictFeedSyncItemXElement = new System.Xml.Linq.XElement(i_ConflictFeedSyncItem.XElement);

                T ImportedConflictFeedSyncItem = new T();

                ImportedConflictFeedSyncItem.Initialize(ImportedConflictFeedSyncItemXElement);

                i_ConflictFeedSyncItem = ImportedConflictFeedSyncItem;
            }

            string Key = System.String.Format
                (
                "{0}{1}{2}",
                i_ConflictFeedSyncItem.Sync.Updates,
                i_ConflictFeedSyncItem.Sync.TopMostHistory.Sequence,
                i_ConflictFeedSyncItem.Sync.TopMostHistory.By
                );

            if (i_ConflictFeedSyncItem.Sync.TopMostHistory.WhenDateTime != null)
                Key += ((System.DateTime)i_ConflictFeedSyncItem.Sync.TopMostHistory.WhenDateTime);

            //  Check if if item already exists in either list
            if (m_ConflictIFeedSyncItemDictionary.ContainsKey(Key))
            {
                if (m_ConflictIFeedSyncItemDictionary.ContainsKey(Key))
                    throw new System.ArgumentException("SyncNode::AddConflictItem (" + Key + ") - item already exists as conflict");
            }

            //  Create "sx:conflicts" element if necessary
            if (m_ConflictsNodeXElement == null)
            {
                m_ConflictsNodeXElement = new System.Xml.Linq.XElement(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.FeedSyncXNamespace + Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.CONFLICTS_ELEMENT_NAME);
                m_XElement.Add(m_ConflictsNodeXElement);
            }

            //  Append conflict node's element to "sx:conflicts" element
            m_ConflictsNodeXElement.Add(i_ConflictFeedSyncItem.XElement);

            //  Add node to list
            m_ConflictIFeedSyncItemDictionary.Add(Key, i_ConflictFeedSyncItem);
        }

        public bool DoesConflictItemNodeExist(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem i_IFeedSyncItem)
        {
            string Key = System.String.Format
                (
                "{0}{1}{2}",
                i_IFeedSyncItem.Sync.Updates,
                i_IFeedSyncItem.Sync.TopMostHistory.Sequence,
                i_IFeedSyncItem.Sync.TopMostHistory.By
                );

            if (i_IFeedSyncItem.Sync.TopMostHistory.WhenDateTime != null)
                Key += ((System.DateTime)i_IFeedSyncItem.Sync.TopMostHistory.WhenDateTime);

            return (m_ConflictIFeedSyncItemDictionary.ContainsKey(Key));
        }

        public string ID
        {
            get
            {
                return m_ID;
            }
        }

        public uint Updates
        {
            get
            {
                return m_Updates;
            }
            set
            {
                if ((value < m_Updates) || (value > (System.Math.Pow(2, 32) - 1)))
                    throw new System.ArgumentException("Invalid value!");

                m_XElement.SetAttributeValue
                    (
                    Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.UPDATES_ATTRIBUTE,
                    value.ToString()
                    );

                m_Updates = value;
            }
        }

        public bool NoConflicts
        {
            get
            {
                return m_NoConflicts;
            }
        }

        public bool Deleted
        {
            get
            {
                return m_Deleted;
            }
            set
            {
                if (value == m_Deleted)
                    return;

                m_XElement.SetAttributeValue
                    (
                    Microsoft.CSA.FeedSync.FeedSyncLibrarySL.Constants.DELETED_ATTRIBUTE,
                    value ? "true" : "false"
                    );

                m_Deleted = value;
            }
        }

        public Microsoft.CSA.FeedSync.FeedSyncLibrarySL.History TopMostHistory
        {
            get
            {
                if (m_HistoryList.Count == 0)
                    return null;

                return m_HistoryList[0];
            }
        }

        public Microsoft.CSA.FeedSync.FeedSyncLibrarySL.History[] Histories
        {
            get
            {
                return m_HistoryList.ToArray();
            }
        }

        public System.Collections.Generic.IList<Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem> Conflicts
        {
            get
            {
                return new System.Collections.Generic.List<Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem>(m_ConflictIFeedSyncItemDictionary.Values);
            }
        }

        public Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem IFeedSyncItem
        {
            get
            {
                return m_IFeedSyncItem;
            }
        }

        public System.Xml.Linq.XElement XElement
        {
            get
            {
                return m_XElement;
            }
        }

        public void Update(System.DateTime? i_WhenDateTime, string i_By)
        {
            this.Update(i_WhenDateTime, i_By, true, false, false);
        }

        public void Update(System.DateTime? i_WhenDateTime, string i_By, bool i_SparsePurge)
        {
            this.Update(i_WhenDateTime, i_By, i_SparsePurge, false, false);
        }

        public void ResolveConflicts(System.DateTime? i_WhenDateTime, string i_By)
        {
            this.Update(i_WhenDateTime, i_By, true, true, false);
        }

        public void ResolveConflicts(System.DateTime? i_WhenDateTime, string i_By, bool i_SparsePurge)
        {
            this.Update(i_WhenDateTime, i_By, i_SparsePurge, true, false);
        }

        public void Delete(System.DateTime? i_WhenDateTime, string i_By)
        {
            this.Update(i_WhenDateTime, i_By, true, false, true);
        }

        public void Delete(System.DateTime? i_WhenDateTime, string i_By, bool i_SparsePurge)
        {
            this.Update(i_WhenDateTime, i_By, i_SparsePurge, false, true);
        }

        private void Update(System.DateTime? i_WhenDateTime, string i_By, bool i_SparsePurge, bool i_ResolveConflicts, bool i_Delete)
        {
            if (!i_WhenDateTime.HasValue && System.String.IsNullOrEmpty(i_By))
                throw new System.ArgumentException("Must have a valid i_WhenDateTime or i_By!");

            if (i_SparsePurge && !System.String.IsNullOrEmpty(i_By))
            {
                for (int Index = m_HistoryList.Count - 1; Index >= 0; --Index)
                {
                    Microsoft.CSA.FeedSync.FeedSyncLibrarySL.History History = m_HistoryList[Index];
                    if (System.String.CompareOrdinal(History.By, i_By) == 0)
                        this.RemoveHistory(History);
                }
            }

            this.Updates++;
            this.Deleted = i_Delete;

            uint Sequence = this.Updates;

            if (!System.String.IsNullOrEmpty(i_By))
            {
                //  Check the histories of the winning item to see if any of them have a sequence
                //  value that is >= sequence assigned above - if so compensate
                foreach (Microsoft.CSA.FeedSync.FeedSyncLibrarySL.History History in m_HistoryList)
                {
                    if ((System.String.CompareOrdinal(History.By, i_By) == 0) && (History.Sequence >= Sequence))
                        Sequence = History.Sequence + 1;
                }


                //  ************************************************************************************
                //  BIG HONKING NOTE:  We must check for conflict items that are subsumed by this update
                //  ************************************************************************************
                System.Collections.Generic.List<Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem> IFeedSyncItemList = new System.Collections.Generic.List<IFeedSyncItem>(m_ConflictIFeedSyncItemDictionary.Values);
                for (int Index = IFeedSyncItemList.Count - 1; Index >= 0; --Index)
                {
                    Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem ConflictIFeedSyncItem = IFeedSyncItemList[Index];
                    if (ConflictIFeedSyncItem.Sync.TopMostHistory.By == i_By)
                        this.RemoveConflict(ConflictIFeedSyncItem);
                }
            }

            Microsoft.CSA.FeedSync.FeedSyncLibrarySL.History History2 = new Microsoft.CSA.FeedSync.FeedSyncLibrarySL.History
                (
                this,
                Sequence,
                i_WhenDateTime,
                i_By
                );

            this.AddHistory(History2);

            bool IgnoreConflictProcessing =
                m_NoConflicts ||
                !i_ResolveConflicts ||
                (m_ConflictIFeedSyncItemDictionary.Count == 0);

            if (IgnoreConflictProcessing)
                return;

            //  *********************************************************************************
            //  BIG HONKING NOTE:  This code resolves all conflicts and does not accomodate for
            //                     selective conflict resolution
            //  *********************************************************************************

            foreach (Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem ConflictIFeedSyncItem in m_ConflictIFeedSyncItemDictionary.Values)
            {
                foreach (Microsoft.CSA.FeedSync.FeedSyncLibrarySL.History ConflictHistory in ConflictIFeedSyncItem.Sync.Histories)
                {
                    bool Subsumed = false;

                    foreach (Microsoft.CSA.FeedSync.FeedSyncLibrarySL.History History in m_HistoryList)
                    {
                        if (ConflictHistory.IsSubsumedBy(History))
                        {
                            Subsumed = true;
                            break;
                        }

                        if ((System.String.CompareOrdinal(History.By, i_By) == 0) && (History.Sequence >= Sequence))
                        {
                            History2.Sequence = History.Sequence + 1;
                            break;
                        }
                    }

                    if (!Subsumed)
                        this.AddConflictHistory(ConflictHistory);
                }
            }

            this.RemoveAllConflictItemNodes();
        }

        internal void AddHistory(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.History i_History)
        {
            if (m_HistoryList.Contains(i_History))
                throw new System.ArgumentException("i_History already added!");

            if (i_History.XElement.Document != m_XElement.Document)
            {
                System.Xml.Linq.XElement ImportedHistoryXElement = new System.Xml.Linq.XElement(i_History.XElement);

                i_History = new Microsoft.CSA.FeedSync.FeedSyncLibrarySL.History(ImportedHistoryXElement);
            }

            if (m_XElement.HasElements)
                m_XElement.FirstNode.AddBeforeSelf(i_History.XElement);
            else
                m_XElement.Add(i_History.XElement);

            //  Make sure new history node is first node in list
            m_HistoryList.Insert
                (
                0,
                i_History
                );

            i_History.Sync = this;
        }

        internal void AddConflictHistory(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.History i_ConflictHistory)
        {
            if (i_ConflictHistory.XElement.Document != m_XElement.Document)
            {
                System.Xml.Linq.XElement ImportedConflictHistoryXElement = new System.Xml.Linq.XElement(i_ConflictHistory.XElement);
                i_ConflictHistory = new Microsoft.CSA.FeedSync.FeedSyncLibrarySL.History(ImportedConflictHistoryXElement);
            }

            //  Insert after topmost history element
            m_XElement.FirstNode.AddAfterSelf(i_ConflictHistory.XElement);

            for (int Index = m_HistoryList.Count - 1; Index >= 0; --Index)
            {
                Microsoft.CSA.FeedSync.FeedSyncLibrarySL.History History = m_HistoryList[Index];
                if (History.IsSubsumedBy(i_ConflictHistory))
                    this.RemoveHistory(History);
            }

            m_HistoryList.Add(i_ConflictHistory);

            i_ConflictHistory.Sync = this;
        }

        internal void RemoveHistory(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.History i_History)
        {
            i_History.XElement.Remove();
            m_HistoryList.Remove(i_History);
        }

        internal void RemoveConflict(Microsoft.CSA.FeedSync.FeedSyncLibrarySL.IFeedSyncItem i_ConflictIFeedSyncItem)
        {
            i_ConflictIFeedSyncItem.XElement.Remove();
            m_ConflictIFeedSyncItemDictionary.Remove(i_ConflictIFeedSyncItem.Sync.ID);
        }

    }
}
